@@ -1,13 +1,11 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 
 """
 Quaternion implementation for use in pymavlink
 """
 
-from __future__ import absolute_import, division, print_function
 
-from builtins import object
 import numpy as np
 from .rotmat import Vector3, Matrix3
 
