@@ -1,11 +1,10 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 
 
 """
 test for trimming under Python 3
 """
 
-from __future__ import absolute_import, print_function
 import unittest
 import os
 import pkg_resources
