@@ -1,10 +1,8 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 '''
 fit best estimate of magnetometer offsets, trying to take into account motor interference
 '''
-from __future__ import print_function
-from builtins import range
 
 from argparse import ArgumentParser
 parser = ArgumentParser(description=__doc__)
