@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 '''
 example program that dumps a Mavlink log file. The log file is
@@ -6,8 +6,6 @@ assumed to be in the format that qgroundcontrol uses, which consists
 of a series of MAVLink packets, each with a 64 bit timestamp
 header. The timestamp is in microseconds since 1970 (unix epoch)
 '''
-from __future__ import print_function
-
 import array
 import fnmatch
 import json
@@ -16,12 +14,6 @@ import struct
 import sys
 import time
 
-# Detect python version
-if sys.version_info[0] < 3:
-    runningPython3 = False
-else:
-    runningPython3 = True
-
 try:
     from pymavlink.mavextra import *
 except:
@@ -50,6 +42,7 @@ parser.add_argument("--no-bad-data", action='store_true', help="Don't output cor
 parser.add_argument("--show-source", action='store_true', help="Show source system ID and component ID")
 parser.add_argument("--show-seq", action='store_true', help="Show sequence numbers")
 parser.add_argument("--show-types", action='store_true', help="Shows all message types available on opened log")
+parser.add_argument("--show-loss", action='store_true', help="Shows changes in lost messages")
 parser.add_argument("--source-system", type=int, default=None, help="filter by source system ID")
 parser.add_argument("--source-component", type=int, default=None, help="filter by source component ID")
 parser.add_argument("--link", type=int, default=None, help="filter by comms link ID")
@@ -76,6 +69,10 @@ if args.profile:
     yappi.start()
 
 if args.format == 'mat':
+    # Check that the mat_file argument has been specified
+    if args.mat_file is None:
+        print("mat_file argument must be specified when mat format is selected")
+        sys.exit(1)
     # Load these modules here, as they're only needed for MAT file creation
     import scipy.io
     import numpy as np
@@ -112,7 +109,6 @@ reduction_count = {}
 
 def reduce_msg(mtype, reduction_ratio):
     '''return True if this msg should be discarded by reduction'''
-    global reduction_count, reduction_msgs, reduction_yes, reduction_no
     if mtype in reduction_no:
         return False
     if not mtype in reduction_yes:
@@ -134,9 +130,8 @@ last_msg_rate_t = {}
 
 def reduce_rate_msg(m, reduction_rate):
     '''return True if this msg should be discarded by reduction'''
-    global last_msg_rate_t
     mtype = m.get_type()
-    if mtype in ['PARM','MSG','FMT','FMTU','MULT','MODE','EVT']:
+    if mtype in ['PARM','MSG','FMT','FMTU','MULT','MODE','EVT','UNIT', 'VER']:
         return False
     t = getattr(m,'_timestamp',None)
     if t is None:
@@ -155,28 +150,10 @@ if args.csv_sep == "tab":
 
 # swiped from DFReader.py
 def to_string(s):
-    """desperate attempt to convert a string regardless of what garbage we get"""
-    try:
-        return s.decode("utf-8")
-    except Exception:
-        pass
-    try:
-        s2 = s.encode("utf-8", "ignore")
-        x = u"%s" % s2
-        return x
-    except Exception:
-        pass
-    # so it's a nasty one. Let's grab as many characters as we can
-    r = ""
-    try:
-        for c in s:
-            r2 = r + c
-            r2 = r2.encode("ascii", "ignore")
-            x = u"%s" % r2
-            r = r2
-    except Exception:
-        pass
-    return r + "_XXX"
+    '''desperate attempt to convert a string regardless of what garbage we get'''
+    if isinstance(s, str):
+        return s
+    return s.decode(errors="backslashreplace")
 
 def match_type(mtype, patterns):
     '''return True if mtype matches pattern'''
@@ -194,29 +171,25 @@ if istlog and args.format == 'csv': # we know our fields from the get-go
         for type in types:
             try:
                 typeClass = "MAVLink_{0}_message".format(type.lower())
-                if runningPython3:
-                    fields += [type + '.' + x for x in inspect.getfullargspec(getattr(mavutil.mavlink, typeClass).__init__).args[1:]]
-                else:
-                    fields += [type + '.' + x for x in inspect.getargspec(getattr(mavutil.mavlink, typeClass).__init__).args[1:]]
+                fields += [type + '.' + x for x in inspect.getfullargspec(getattr(mavutil.mavlink, typeClass).__init__).args[1:]]
                 offsets[type] = currentOffset
                 currentOffset += len(fields)
             except IndexError:
-                quit()
+                sys.exit(1)
+            except AttributeError:
+                print("Message type '%s' not found" % (type))
+                sys.exit(1)
     except TypeError:
         print("You must specify a list of message types if outputting CSV format via the --types argument.")
-        exit()
+        sys.exit(1)
 
     # The first line output are names for all columns
-    csv_out = ["" for x in fields]
     print(args.csv_sep.join(fields))
 
-if isbin and args.format == 'csv': # need to accumulate columns from message
+if (isbin or islog) and args.format == 'csv': # need to accumulate columns from message
     if types is None or len(types) != 1:
         print("Need exactly one type when dumping CSV from bin file")
-        quit()
-
-# Track the last timestamp value. Used for compressing data for the CSV output format.
-last_timestamp = None
+        sys.exit(1)
 
 # Track types found
 available_types = set()
@@ -232,26 +205,27 @@ if types is not None and hasattr(mlog, 'name_to_id'):
                 match_types = []
             match_types.append(k)
 
-if isbin and args.format == 'csv':
+if (isbin or islog) and args.format == 'csv':
+    # Make sure the specified type was found
+    if match_types is None:
+        print("Specified type '%s' not found in log file" % (types[0]))
+        sys.exit(1)
     # we need FMT messages for column headings
     match_types.append("FMT")
 
+last_loss = 0
+
 # Keep track of data from the current timestep. If the following timestep has the same data, it's stored in here as well. Output should therefore have entirely unique timesteps.
 MAT = {}    # Dictionary to hold output data for 'mat' format option
 while True:
     m = mlog.recv_match(blocking=args.follow, type=match_types)
     if m is None:
-        # write the final csv line before exiting
-        if args.format == 'csv' and csv_out:
-          csv_out[0] = "{:.8f}".format(last_timestamp)
-          print(args.csv_sep.join(csv_out))
         break
     m_type = m.get_type()
     available_types.add(m_type)
-    if isbin and m_type == "FMT" and args.format == 'csv':
+    if (isbin or islog) and m_type == "FMT" and args.format == 'csv':
         if m.Name == types[0]:
             fields += m.Columns.split(',')
-            csv_out = ["" for x in fields]
             print(args.csv_sep.join(fields))
 
     if args.reduce and reduce_msg(m_type, args.reduce):
@@ -259,11 +233,14 @@ while True:
 
     if args.reduce_rate > 0 and reduce_rate_msg(m, args.reduce_rate):
         continue
-    
+
     if output is not None:
         if (isbin or islog) and m_type == "FMT":
             output.write(m.get_msgbuf())
             continue
+        if (isbin or islog) and m_type in ["FMTU", "MULT", "UNIT"]:
+            output.write(m.get_msgbuf())
+            continue
         if (isbin or islog) and (m_type == "PARM" and args.parms):
             output.write(m.get_msgbuf())
             continue
@@ -273,7 +250,7 @@ while True:
             continue
 
     if not mavutil.evaluate_condition(args.condition, mlog.messages) and (
-            not (m_type in ['FMT', 'FMTU', 'MULT','PARM','MODE'] and args.meta)):
+            not (m_type in ['FMT', 'FMTU', 'MULT', 'PARM', 'MODE', 'UNIT', 'VER','CMD','MAVC','MSG','EV'] and args.meta)):
         continue
     if args.source_system is not None and args.source_system != m.get_srcSystem():
         continue
@@ -344,27 +321,13 @@ while True:
     # CSV format outputs columnar data with a user-specified delimiter
     elif args.format == 'csv':
         data = m.to_dict()
-
-        # If this message has a duplicate timestamp, copy its data into the existing data list. Also
-        # do this if it's the first message encountered.
-        if timestamp == last_timestamp or last_timestamp is None:
-            if isbin:
-                newData = [str(data[y]) if y != "timestamp" else "" for y in fields]
-            else:
-                newData = [str(data[y.split('.')[-1]]) if y.split('.')[0] == m_type and y.split('.')[-1] in data else "" for y in fields]
-
-            for i, val in enumerate(newData):
-                if val:
-                    csv_out[i] = val
-
-        # Otherwise if this is a new timestamp, print out the old output data, and store the current message for later output.
+        if isbin or islog:
+            csv_out = [str(data[y]) if y != "timestamp" else "" for y in fields]
         else:
-            csv_out[0] = "{:.8f}".format(last_timestamp)
-            print(args.csv_sep.join(csv_out))
-            if isbin:
-                csv_out = [str(data[y]) if y != "timestamp" else "" for y in fields]
-            else:
-                csv_out = [str(data[y.split('.')[-1]]) if y.split('.')[0] == m_type and y.split('.')[-1] in data else "" for y in fields]
+            csv_out = [str(data[y.split('.')[-1]]) if y.split('.')[0] == m_type and y.split('.')[-1] in data else "" for y in fields]
+        csv_out[0] = "{:.8f}".format(timestamp)
+        print(args.csv_sep.join(csv_out))
+
     # MAT format outputs data to a .mat file specified through the
     # --mat_file option
     elif args.format == 'mat':
@@ -394,6 +357,9 @@ while True:
     elif args.verbose and istlog:
         mavutil.dump_message_verbose(sys.stdout, m)
         print("")
+    elif args.verbose and hasattr(m,"dump_verbose"):
+        m.dump_verbose(sys.stdout)
+        print("")
     else:
         # Otherwise we output in a standard Python dict-style format
         s = "%s.%02u: %s" % (time.strftime("%Y-%m-%d %H:%M:%S",
@@ -404,9 +370,10 @@ while True:
         if args.show_seq:
             s += " seq=%u" % m.get_seq()
         print(s)
-
-    # Update our last timestamp value.
-    last_timestamp = timestamp
+    if args.show_loss:
+        if last_loss != mlog.mav_loss:
+            print("lost %d messages" % (mlog.mav_loss - last_loss))
+            last_loss = mlog.mav_loss
 
 # Export the .mat file
 if args.format == 'mat':
