@@ -1,10 +1,8 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 '''
 Summarize MAVLink logs. Useful for identifying which log is of interest in a large set.
 '''
-from __future__ import print_function
-from builtins import object
 
 import glob
 import time
